/*
 * House.cpp: A class for drawing the ground.
 */


#include "House.h"
#include "libtarga.h"
#include <stdio.h>
#include <GL/glu.h>


// Destructor
House::~House(void)
{
    if ( initialized )
    {
	glDeleteLists(house_list, 1);
    }
}


// Initializer. Returns false if something went wrong, like not being able to
// load the texture.
bool
House::Initialize(void)
{
    ubyte   *image_data;
    int	    image_height, image_width;


    // Load the image for the texture. The texture file has to be in
    // a place where it will be found.
    if ( ! ( image_data = (ubyte*)tga_load("wood.tga", &image_width,
					   &image_height, TGA_TRUECOLOR_24) ) )
    {
	fprintf(stderr, "Ground::Initialize: Couldn't load wood.tga\n");
	return false;
    }

    // This creates a texture object and binds it, so the next few operations
    // apply to this texture.
    glGenTextures(1, &wood_texture_obj);
    glBindTexture(GL_TEXTURE_2D, wood_texture_obj);

    // This sets a parameter for how the texture is loaded and interpreted.
    // basically, it says that the data is packed tightly in the image array.
    glPixelStorei(GL_UNPACK_ALIGNMENT, 1);

    // This sets up the texture with high quality filtering. First it builds
    // mipmaps from the image data, then it sets the filtering parameters
    // and the wrapping parameters. We want the grass to be repeated over the
    // ground.
    gluBuild2DMipmaps(GL_TEXTURE_2D,3, image_width, image_height, 
		      GL_RGB, GL_UNSIGNED_BYTE, image_data);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER,
		    GL_NEAREST_MIPMAP_LINEAR);


    // This says what to do with the texture. Modulate will multiply the
    // texture by the underlying color.
    glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE); 

	if ( ! ( image_data = (ubyte*)tga_load("door.tga", &image_width,
					   &image_height, TGA_TRUECOLOR_24) ) )
    {
	fprintf(stderr, "Ground::Initialize: Couldn't load door.tga\n");
	return false;
    }
	glGenTextures(1, &door_texture_obj);
    glBindTexture(GL_TEXTURE_2D, door_texture_obj);
	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
	gluBuild2DMipmaps(GL_TEXTURE_2D,3, image_width, image_height, 
		      GL_RGB, GL_UNSIGNED_BYTE, image_data);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER,
		    GL_NEAREST_MIPMAP_LINEAR);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);

	if ( ! ( image_data = (ubyte*)tga_load("shingle.tga", &image_width,
					   &image_height, TGA_TRUECOLOR_24) ) )
    {
	fprintf(stderr, "Ground::Initialize: Couldn't load shingle.tga\n");
	return false;
    }
	glGenTextures(1, &shingle_texture_obj);
    glBindTexture(GL_TEXTURE_2D, shingle_texture_obj);
	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
	gluBuild2DMipmaps(GL_TEXTURE_2D,3, image_width, image_height, 
		      GL_RGB, GL_UNSIGNED_BYTE, image_data);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER,
		    GL_NEAREST_MIPMAP_LINEAR);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);

	// Now do the geometry. Create the display list.
    house_list = glGenLists(1);
    glNewList(house_list, GL_COMPILE);
	glColor3f(1.0f,1.0f,1.0f);

	// Turn on texturing and bind the grass texture.
		glEnable(GL_TEXTURE_2D);
		glBindTexture(GL_TEXTURE_2D, door_texture_obj);

	// Draw the ground as a quadrilateral, specifying texture coordinates.
	glBegin(GL_QUADS);				
		glNormal3f(1.0f, 0.0f, 0.0f);
		glTexCoord2f(0.0f, 0.0f);
		glVertex3f(5.0f, -5.0f, 0.0f);
		glTexCoord2f(1.0f, 0.0f);
		glVertex3f(5.0f, 5.0f, 0.0f);
		glTexCoord2f(1.0f, 1.0f);
		glVertex3f(5.0f, 5.0f, 6.0f);
		glTexCoord2f(0.0f, 1.0f);
		glVertex3f(5.0f, -5.0f, 6.0f);
	glEnd();
	
	glBindTexture(GL_TEXTURE_2D, wood_texture_obj);
	
	glBegin(GL_QUADS);

		

		glNormal3f(-1.0f, 0.0f, 0.0f);
		glTexCoord2f(0.0f, 0.0f);
		glVertex3f(-5.0f, 5.0f, 0.0f);
		glTexCoord2f(1.0f, 0.0f);
		glVertex3f(-5.0f, -5.0f, 0.0f);
		glTexCoord2f(1.0f, 1.0f);
		glVertex3f(-5.0f, -5.0f, 6.0f);
		glTexCoord2f(0.0f, 1.0f);
		glVertex3f(-5.0f, 5.0f, 6.0f);

		
		glNormal3f(0.0f, 1.0f, 0.0f);
		glTexCoord2f(0.0f, 0.0f);
		glVertex3f(5.0f, 5.0f, 0.0f);
		glTexCoord2f(1.0f, 0.0f);
		glVertex3f(-5.0f, 5.0f, 0.0f);
		glTexCoord2f(1.0f, 1.0f);
		glVertex3f(-5.0f, 5.0f, 6.0f);
		glTexCoord2f(0.0f, 1.0f);
		glVertex3f(5.0f, 5.0f, 6.0f);

		
		glNormal3f(0.0f, -1.0f, 0.0f);
		glTexCoord2f(0.0f, 0.0f);
		glVertex3f(-5.0f, -5.0f, 0.0f);
		glTexCoord2f(1.0f, 0.0f);
		glVertex3f(5.0f, -5.0f, 0.0f);
		glTexCoord2f(1.0f, 1.0f);
		glVertex3f(5.0f, -5.0f, 6.0f);
		glTexCoord2f(0.0f, 1.0f);
		glVertex3f(-5.0f, -5.0f, 6.0f);

	glEnd();

	glBegin(GL_TRIANGLES);
		
		glNormal3f(1.0f, 0.0f, 0.0f);
		glTexCoord2f(0.0f, 0.0f);
		glVertex3f(5.0f, -5.0f, 6.0f);
		glTexCoord2f(1.0f, 0.0f);
		glVertex3f(5.0f, 5.0f, 6.0f);
		glTexCoord2f(0.5f, 0.5f);
		glVertex3f(5.0f, 0.0f, 9.0f);

		glNormal3f(-1.0f, 0.0f, 0.0f);
		
		glTexCoord2f(1.0f, 0.0f);
		glVertex3f(-5.0f, 5.0f, 6.0f);
		glTexCoord2f(0.0f, 0.0f);
		glVertex3f(-5.0f, -5.0f, 6.0f);
		glTexCoord2f(0.5f, 0.5f);
		glVertex3f(-5.0f, 0.0f, 9.0f);	
		
	glEnd();

	glBindTexture(GL_TEXTURE_2D, shingle_texture_obj);

	glBegin(GL_QUADS);
		glNormal3f(0.0f, 0.375f, 0.625f);
		glTexCoord2f(0.0f, 0.0f);
		glVertex3f(5.0f, 5.0f, 6.0f);
		glTexCoord2f(5.0f, 0.0f);
		glVertex3f(-5.0f, 5.0f, 6.0f);
		glTexCoord2f(5.0f,5.0f);
		glVertex3f(-5.0f, 0.0f, 9.0f);
		glTexCoord2f(0.0f,5.0f);
		glVertex3f(5.0f, 0.0f, 9.0f);

		glNormal3f(0.0f, -0.375f, 0.625f);
		glTexCoord2f(0.0f, 0.0f);
		glVertex3f(-5.0f, -5.0f, 6.0f);
		glTexCoord2f(5.0f, 0.0f);
		glVertex3f(5.0f, -5.0f, 6.0f);
		glTexCoord2f(5.0f,5.0f);
		glVertex3f(5.0f, 0.0f, 9.0f);
		glTexCoord2f(0.0f,5.0f);
		glVertex3f(-5.0f, 0.0f, 9.0f);
		
		
	glEnd();


	glDisable(GL_TEXTURE_2D);
	
    glEndList();


    // We only do all this stuff once, when the GL context is first set up.
    initialized = true;

    return true;
}


// Draw just calls the display list we set up earlier.
void
House::Draw(void)
{
	glPushMatrix();
	glTranslatef(25.0f, -25.0f, 0.0f);
	glCallList(house_list);
	glPopMatrix();
		
}